@@ -24,7 +24,7 @@
 //
 
 using Gtk;
-using GtkSourceView;
+using Mono.TextEditor;
 using System;
 using System.IO;
 using System.Collections.Generic;
@@ -37,58 +37,56 @@ using MonoDevelop.Database.Sql;
 
 namespace MonoDevelop.Database.Components
 {
+	//TODO: use the abstracted MD source editor widget
+	//TODO: remove gtksourceview-sharp as dependency + from configure.in
+	[System.ComponentModel.Category("widget")]
+	[System.ComponentModel.ToolboxItem(true)]
 	public partial class SqlEditorWidget : Bin
 	{
 		public event EventHandler TextChanged;
 		
-		private SourceView sourceView;
+		private Mono.TextEditor.TextEditor sourceView;
 		
 		public SqlEditorWidget()
 		{
 			this.Build();
 			
-			SourceLanguagesManager lm = new SourceLanguagesManager ();
-			SourceLanguage lang = lm.GetLanguageFromMimeType ("text/x-sql");
-			SourceBuffer buf = new SourceBuffer (lang);
-			buf.Highlight = true;
-			sourceView = new SourceView (buf);
-			sourceView.ShowLineNumbers = true;
+			sourceView = new Mono.TextEditor.TextEditor ();
+			sourceView.Document.MimeType = "text/x-sql";
+			// TODO: Set styling ?
+		//	sourceView.Options = new MonoDevelop.SourceEditor.StyledSourceEditorOptions (null);
+		//	sourceView.ShowLineNumbers = true;
 			
-			sourceView.Buffer.Changed += new EventHandler (BufferChanged);
-			sourceView.PopulatePopup += new PopulatePopupHandler (OnPopulatePopup);
+			sourceView.Document.TextReplaced += BufferChanged;
+			sourceView.TextViewMargin.ButtonPressed += delegate (object s, MarginMouseEventArgs args) {
+				if (args.Button == 3) {
+					IdeApp.CommandService.ShowContextMenu ("/MonoDevelop/Database/ContextMenu/SqlEditor");
+				}
+			};
 			
 			scrolledwindow.Add (sourceView);
 			ShowAll ();
 		}
-
-		protected virtual void OnPopulatePopup (object sender, PopulatePopupArgs args)
-		{
-			CommandEntrySet cset = IdeApp.CommandService.CreateCommandEntrySet ("/MonoDevelop/Database/ContextMenu/SqlEditor");
-			if (cset.Count > 0) {
-				cset.AddItem (Command.Separator);
-				IdeApp.CommandService.InsertOptions (args.Menu, cset, 0);
-			}
-		}
 		
-		private void BufferChanged (object sender, EventArgs args)
+		private void BufferChanged (object sender, ReplaceEventArgs args)
 		{
 			if (TextChanged != null)
 				TextChanged (this, EventArgs.Empty);
 		}
 		
 		public string Text {
-			get { return sourceView.Buffer.Text; }
+			get { return sourceView.Document.Text; }
 			set {
 				if (value == null)
-					sourceView.Buffer.Text = String.Empty;
+					sourceView.Document.Text = String.Empty;
 				else
-					sourceView.Buffer.Text = value;
+					sourceView.Document.Text = value;
 			}
 		}
 		
 		public bool Editable {
-			get { return sourceView.Editable; }
-			set { sourceView.Editable = value; }
+			get { return !sourceView.Document.ReadOnly; }
+			set { sourceView.Document.ReadOnly = !value; }
 		}
 		
 		[CommandHandler (SqlEditorCommands.ImportFromFile)]
@@ -142,9 +140,8 @@ namespace MonoDevelop.Database.Components
 
 			if (dlg.Run () == (int)ResponseType.Accept) {
 				if (File.Exists (dlg.Filename)) {
-					bool overwrite = Services.MessageService.AskQuestion (
-						AddinCatalog.GetString ("Are you sure you want to overwrite the file '{0}'?", dlg.Filename), 
-						AddinCatalog.GetString ("Overwrite?"));
+					bool overwrite = MessageService.Confirm (
+						AddinCatalog.GetString ("Are you sure you want to overwrite the file '{0}'?", dlg.Filename), AlertButton.OverwriteFile);
 					if (overwrite) {
 						using (FileStream stream = File.Open (dlg.Filename, FileMode.Create)) {
 							using (StreamWriter writer = new StreamWriter (stream)) {
