@@ -6,7 +6,7 @@
 //	Ben Motmans  <ben.motmans@gmail.com>
 //
 // Copyright (C) 2005 Mosaix Communications, Inc.
-// Copyright (c) 2007 Ben Motmans
+// Copyright (c) 2007-2008 Ben Motmans
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -35,15 +35,31 @@ using MySql.Data.MySqlClient;
 using System.Collections.Generic;
 using MonoDevelop.Core;
 
-namespace MonoDevelop.Database.Sql
+namespace MonoDevelop.Database.Sql.MySql
 {
-	public class MySqlSchemaProvider : AbstractSchemaProvider
+	public class MySqlSchemaProvider : AbstractEditSchemaProvider
 	{
 		public MySqlSchemaProvider (IConnectionPool connectionPool)
 			: base (connectionPool)
 		{
+			AddSupportedSchemaActions (SchemaType.Database, SchemaActions.All);
+			AddSupportedSchemaActions (SchemaType.Table, SchemaActions.Create | SchemaActions.Drop | SchemaActions.Rename | SchemaActions.Schema);
+			AddSupportedSchemaActions (SchemaType.View, SchemaActions.All);
+			AddSupportedSchemaActions (SchemaType.TableColumn, SchemaActions.All);
+			AddSupportedSchemaActions (SchemaType.Trigger, SchemaActions.All);
+			AddSupportedSchemaActions (SchemaType.PrimaryKeyConstraint, SchemaActions.Create | SchemaActions.Drop | SchemaActions.Rename | SchemaActions.Schema);
+			AddSupportedSchemaActions (SchemaType.ForeignKeyConstraint, SchemaActions.Create | SchemaActions.Drop | SchemaActions.Rename | SchemaActions.Schema);
+			AddSupportedSchemaActions (SchemaType.CheckConstraint, SchemaActions.Create | SchemaActions.Drop | SchemaActions.Rename | SchemaActions.Schema);
+			AddSupportedSchemaActions (SchemaType.UniqueConstraint, SchemaActions.Create | SchemaActions.Drop | SchemaActions.Rename | SchemaActions.Schema);
+			AddSupportedSchemaActions (SchemaType.Constraint, SchemaActions.Create | SchemaActions.Drop | SchemaActions.Rename | SchemaActions.Schema);
+			AddSupportedSchemaActions (SchemaType.User, SchemaActions.Schema);
+			
+			if (connectionPool.DatabaseVersion.Major > 4) {
+				AddSupportedSchemaActions (SchemaType.Procedure, SchemaActions.All);
+				AddSupportedSchemaActions (SchemaType.ProcedureParameter, SchemaActions.Schema);
+			}
 		}
-		
+
 		public override DatabaseSchemaCollection GetDatabases ()
 		{
 			DatabaseSchemaCollection databases = new DatabaseSchemaCollection ();
@@ -394,9 +410,9 @@ namespace MonoDevelop.Database.Sql
 							
 							ConstraintSchema constraint = null;
 							if (key.Contains ("PRI")) {
-								constraint = GetNewPrimaryKeyConstraintSchema ("pk_" + column.Name);
+								constraint = CreatePrimaryKeyConstraintSchema ("pk_" + column.Name);
 							} else if (key.Contains ("UNI")) {
-								constraint = GetNewUniqueConstraintSchema ("uni_" + column.Name);
+								constraint = CreateUniqueConstraintSchema ("uni_" + column.Name);
 							} else {
 								continue;
 							}
@@ -556,6 +572,69 @@ namespace MonoDevelop.Database.Sql
 			return dts;
 		}
 		
+		//http://dev.mysql.com/doc/refman/5.0/en/charset-mysql.html
+		public MySqlCharacterSetSchemaCollection GetCharacterSets ()
+		{
+			MySqlCharacterSetSchemaCollection characterSets = new MySqlCharacterSetSchemaCollection ();
+
+			IPooledDbConnection conn = connectionPool.Request ();
+			IDbCommand command = conn.CreateCommand ("SHOW CHARACTER SET;");
+			try {
+				using (command) {
+					using (IDataReader r = command.ExecuteReader ()) {
+						while (r.Read ()) {
+							MySqlCharacterSetSchema charset = new MySqlCharacterSetSchema (this);
+							charset.Name = r.GetString (0);
+							charset.Comment = r.GetString (1);
+							charset.DefaultCollactionName = r.GetString (2);
+							charset.MaxLength = r.GetInt32 (3);
+		
+							characterSets.Add (charset);
+						}
+		
+						r.Close ();
+					}
+				}
+			} catch (Exception e) {
+				QueryService.RaiseException (e);
+			}
+			conn.Release ();
+
+			return characterSets;
+		}
+		
+		public MySqlCollationSchemaCollection GetCollations (MySqlCharacterSetSchema characterSet)
+		{
+			MySqlCollationSchemaCollection collations = new MySqlCollationSchemaCollection ();
+
+			IPooledDbConnection conn = connectionPool.Request ();
+			IDbCommand command = conn.CreateCommand (String.Format ("SHOW COLLATION LIKE '{0}%';", characterSet.Name));
+			try {
+				using (command) {
+					using (IDataReader r = command.ExecuteReader ()) {
+						while (r.Read ()) {
+							MySqlCollationSchema collation = new MySqlCollationSchema (this);
+							collation.Name = r.GetString (0);
+							collation.CharacterSetName = r.GetString (1);
+							collation.Id = r.GetInt32 (2);
+							collation.IsDefaultCollation = r.GetBoolean (3);
+							collation.IsCompiled = r.GetBoolean (4);
+							collation.SortLength = r.GetInt32 (5);
+		
+							collations.Add (collation);
+						}
+		
+						r.Close ();
+					}
+				}
+			} catch (Exception e) {
+				QueryService.RaiseException (e);
+			}
+			conn.Release ();
+
+			return collations;
+		}
+		
 		//http://dev.mysql.com/doc/refman/5.1/en/create-database.html
 		public override void CreateDatabase (DatabaseSchema database)
 		{
@@ -740,27 +819,28 @@ namespace MonoDevelop.Database.Sql
 		}
 		
 		//http://dev.mysql.com/doc/refman/5.1/en/alter-database.html
-		public override void AlterDatabase (DatabaseSchema database)
+		public override void AlterDatabase (DatabaseAlterSchema database)
 		{
 			throw new NotImplementedException ();
 		}
 
 		//http://dev.mysql.com/doc/refman/5.1/en/alter-table.html
-		public override void AlterTable (TableSchema table)
+		public override void AlterTable (TableAlterSchema table)
 		{
 			throw new NotImplementedException ();
 		}
 
 		//http://dev.mysql.com/doc/refman/5.1/en/alter-view.html
-		public override void AlterView (ViewSchema view)
+		public override void AlterView (ViewAlterSchema view)
 		{
-			ExecuteNonQuery (view.Definition);
+//TODO: fixme
+//			ExecuteNonQuery (view.Definition);
 		}
 
 		//http://dev.mysql.com/doc/refman/5.1/en/alter-procedure.html
-		public override void AlterProcedure (ProcedureSchema procedure)
+		public override void AlterProcedure (ProcedureAlterSchema procedure)
 		{
-			ExecuteNonQuery (procedure.Definition);
+			//ExecuteNonQuery (procedure.Definition); //TODO: fixme
 		}
 		
 		//http://dev.mysql.com/doc/refman/5.1/en/drop-database.html
@@ -839,16 +919,24 @@ namespace MonoDevelop.Database.Sql
 			user.Name = name;
 		}
 		
-		public override string GetViewAlterStatement (ViewSchema view)
+		public override DatabaseSchema CreateDatabaseSchema (string name)
 		{
-			return String.Concat ("DROP VIEW IF EXISTS ", view.Name, "; ", Environment.NewLine, view.Definition); 
+			MySqlDatabaseSchema schema = new MySqlDatabaseSchema (this);
+			schema.Name = name;
+			return schema;
 		}
 		
-		public override string GetProcedureAlterStatement (ProcedureSchema procedure)
-		{
-			return String.Concat ("DROP PROCEDURE IF EXISTS ", procedure.Name, "; ", Environment.NewLine, procedure.Definition);
-		}
+//		public override string GetViewAlterStatement (ViewAlterSchema view)
+//		{
+//			return String.Concat ("DROP VIEW IF EXISTS ", view.Name, "; ", Environment.NewLine, view.Definition); 
+//		}
+//		
+//		public override string GetProcedureAlterStatement (ProcedureAlterSchema procedure)
+//		{
+//			return String.Concat ("DROP PROCEDURE IF EXISTS ", procedure.Name, "; ", Environment.NewLine, procedure.Definition);
+//		}
 		
+//TODO: remove this and use the Version provided by the connection pool
 		private int GetMainVersion (IDbCommand command)
 		{
 			string str = (command.Connection as MySqlConnection).ServerVersion;
@@ -886,4 +974,4 @@ namespace MonoDevelop.Database.Sql
 			}
 		}
 	}
-}
+}
\ No newline at end of file
